from colorama import Fore, Style
import aiofiles
import random
import yaml
import aiohttp
import sys
import re
from fake_useragent import UserAgent

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def dnsdumpster(domain, session, configs, username, args):
    
    try:
        
        try:
            
            async with aiofiles.open(configs, "r") as streamr:
                
                data = yaml.safe_load(await streamr.read())
                
                if "dnsdumpster" not in data:
                    
                    return []
                
                rand = data.get("dnsdumpster", [])
                
                if rand is None:
                    
                    return []
                
                cooktoken = random.choice(data.get("dnsdumpster", []))
                
                randcookie, randtoken = cooktoken.split(":")
                
                if randcookie is None or randtoken is None:
                    
                    return []
                
        except yaml.YAMLError as e:
            
            if args.show_key_info: 
                
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)  
                      
                else:
                    
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                               
        except TypeError as e:
            
            pass 
        
        except yaml.YAMLObject as e:
            
            if args.show_key_info: 
                
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)    
                    
                else:
                    
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr)    
                    
        except KeyboardInterrupt as e:
            quit()
            
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in dnsdumpster reader block: {e}, {type(e)}", file=sys.stderr) 
        try:
            
            dnsdumpsters = []
            
            url = f"https://dnsdumpster.com/"
                
            auth = {
                'Cookie': f'csrftoken={randcookie}',
                'User-Agent': UserAgent().random,
                'Referer': 'https://dnsdumpster.com/'
                }
            
            data = {
                'csrfmiddlewaretoken': f'{randtoken}',
                'targetip': domain,
                'user': 'free'
               }
            proxy = args.proxy if args.proxy else None
            async with session.post(url, headers=auth, data=data, timeout=args.timeout, proxy=proxy, ssl=False) as response:
                
                data = await response.text()
                
                if response.status != 200:
                    return []
                
                pattern = re.compile(rf"\b[a-zA-Z0-9]+\b\.{domain}\b")
                
                subdomains = pattern.findall(data)
                
                subdomains1 = re.findall(fr'(?<=http://)[\w.-]+(?=\.{domain})', data) #re from stf
                
                for subs in subdomains1:
                    if not subs.endswith(f"{domain}"): 
                        dnsdumpsters.append(f"{subs}.{domain}") 
                    else:
                        dnsdumpsters.append(subs) 
                                        
                for subdomain in subdomains:
                    if not subdomain.endswith(f"{domain}"): 
                        dnsdumpsters.append(f"{subdomain}.{domain}")
                    else:
                        dnsdumpsters.append(subdomain)
                        
                return dnsdumpsters
            
        except aiohttp.ServerConnectionError as e:
            
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Dnsdumpster API, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Dnsdumpster API, due to Serverside Error", file=sys.stderr)
                
        except aiohttp.ClientConnectionError as e:
            
            if args.show_timeout_info:
                 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Dnsdumpster API, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Dnsdumpster API, due to Clientside connection Error", file=sys.stderr) 
        
        except TimeoutError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Dnsdumpster API, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Dnsdumpster API, due to Timeout Error", file=sys.stderr)
                    
        except KeyboardInterrupt as e:
            quit()
                           
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api dnsdumpster req block: {e}, due to: {type(e)}{reset}", file=sys.stderr)
                    
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api dnsdumpster main block: {e}, due to: {type(e)}", file=sys.stderr)
                



from colorama import Fore, Style
import aiofiles
import random
import yaml
import aiohttp
import sys

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def rapidapi(domain, session, configs, username, args):
    
    try:
        
        try:
            
            async with aiofiles.open(configs, "r") as streamr:
                
                data = yaml.safe_load(await streamr.read())
                
                if "rapidapi" not in data:
                    
                    return
                
                elif "whoisxmlapi" not in data:
                    
                    return
                else:
                    
                    pass
                
                randomrapidkey = random.choice(data.get("rapidapi", []))
                
                randomwhoiskey = random.choice(data.get("whoisxmlapi", []))
                
                if randomrapidkey is None:
                    
                    return
                
                if randomwhoiskey is None:
                    
                    return 
                
        except yaml.YAMLError as e:
            
            if args.show_key_info: 
                
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)  
                      
                else:
                    
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                               
        except TypeError as e:
            
            pass 
            
        except KeyboardInterrupt as e:
            quit()
        
        except yaml.YAMLObject as e:
            
            if args.show_key_info: 
                
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)    
                    
                else:
                    
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr)
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception at rapidapi reader block: {e}, {type(e)}{reset}", file=sys.stderr)
                    
        try:
            
            rapids = []
            
            url = "https://subdomains-lookup.p.rapidapi.com/api/v1"
            
            params = {"domainName": domain,"apiKey": randomwhoiskey ,"outputFormat":"JSON"}
            
            auth = {
	                "X-RapidAPI-Key": randomrapidkey,
	                "X-RapidAPI-Host": "subdomains-lookup.p.rapidapi.com"
                 }
            proxy = args.proxy if args.proxy else None
            async with session.get(url, params=params, headers=auth,timeout=args.timeout, proxy=proxy, ssl=False) as response:
                
                data = await response.json()
                
                if response.status != 200:
                    
                    if args.show_key_info:
                        
                        if not args.no_color:
                        
                            print(f"[{bold}{red}ALERT{reset}]: Rapidadpi blocking our request, {username} please check your api usage for this keys: {randomwhoiskey}, {randomrapidkey}", file=sys.stderr)
                            
                        else:
                            
                            print(f"[ALERT]: Rapidapi blocking our request, {username} please check your api usage for this keys: {randomwhoiskey}, {randomrapidkey}", file=sys.stderr)
                    
                    return []
                
                subdomains = data.get('result', {}).get('records', [])
                
                for sub in subdomains:
                    
                    subdomain = sub.get("domain")
                    
                    if subdomain:
                        
                        rapids.append(subdomain)
                        
                return rapids
        
        except aiohttp.ServerConnectionError as e:
            
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for RapidAPI API, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for RapidAPI API, due to Serverside Error", file=sys.stderr)
                
        except aiohttp.ClientConnectionError as e:
            
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for RapidAPI API, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for RapidAPI API, due to Clientside connection Error", file=sys.stderr) 
             
        except KeyboardInterrupt as e:
            quit()
        
        except TimeoutError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for RapidAPI API, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for RapidAPI API, due to Timeout Error", file=sys.stderr)
                    
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api rapid req block: {e}, due to: {type(e)}{reset}", file=sys.stderr)
            
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api rapid main block: {e}, due to: {type(e)}{reset}", file=sys.stderr)
            


#!/usr/bin/env python3
from colorama import Fore,Back,Style
from art import *
import random

red =  Fore.RED
green = Fore.GREEN
magenta = Fore.MAGENTA
cyan = Fore.CYAN
mixed = Fore.RED + Fore.BLUE
blue = Fore.BLUE
yellow = Fore.YELLOW
white = Fore.WHITE
reset = Style.RESET_ALL
bold = Style.BRIGHT
colors = [ green, cyan, blue, white, magenta]
random_color = random.choice(colors)


def banner():
    tool_name = "subdominator"
    fonts = ["big", "ogre", "shadow", "script",  "graffiti", "slant"]
    selected_font = random.choice(fonts)
    banner = text2art(f"{tool_name}", font=selected_font)
    banner = f"""{bold}{random_color}{banner}{reset}
                     {bold}{white}@RevoltSecurities{reset}\n"""
    return banner




from colorama import Fore, Style
import aiohttp
import sys

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def hackertarget(domain, session, args):
    try:
        hackertargets =[]
        url = f"https://api.hackertarget.com/hostsearch/?q={domain}"
        proxy= args.proxy if args.proxy else None
        async with session.get(url, timeout=args.timeout, proxy=proxy, ssl=False) as response:
            if response.status !=200:
                return
            responsed = await response.text()
            data = responsed.splitlines()
            for subdomain in data:
                if "API count exceeded - Increase Quota with Membership" in subdomain:
                    pass
                else:
                    subdomain = subdomain.split(",")[0]
                    hackertargets.append(subdomain)
        return hackertargets
    
    except aiohttp.ServerConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Hackertarget API, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Hackertarget API, due to Serverside Error", file=sys.stderr)
                    
    except aiohttp.ClientConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Hackertarget API, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Hackertarget API, due to Clientside connection Error", file=sys.stderr)
                    
    except TimeoutError as e:
        if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Hackertarget API, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Hackertarget API, due to Timeout Error", file=sys.stderr)
                    
    except KeyboardInterrupt as e:
        quit()
        
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}INFO{reset}]: {bold}{white}Exception at hackertarget: {e}, {type(e)}{reset}", file=sys.stderr)
            


from colorama import Fore, Style
import aiohttp
import sys
from bs4 import  XMLParsedAsHTMLWarning, MarkupResemblesLocatorWarning, BeautifulSoup
from fake_useragent import UserAgent
import warnings
from subdominator.modules.utils import check_subdomain
bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL
async def shodanx(domain, session, args):
    try:
        shodanxs = []
        parsed_domain = check_subdomain(domain)
        if parsed_domain.subdomain:
            return []
        url = f"https://www.shodan.io/domain/{domain}"
        auth = {
            "User-Agent": UserAgent().random
        }
        proxy= args.proxy if args.proxy else None
        async with session.get(url, headers=auth, timeout=aiohttp.ClientTimeout(total=args.timeout,connect=args.timeout,sock_read=120,sock_connect=args.timeout), proxy=proxy, ssl=False) as response:
            if response.status != 200:
                return []
            data = await response.text()
            with warnings.catch_warnings():
                warnings.filterwarnings("ignore", category=UserWarning)
                warnings.filterwarnings('ignore', category=XMLParsedAsHTMLWarning)
                warnings.filterwarnings("ignore", category=MarkupResemblesLocatorWarning)
                soup = BeautifulSoup(data, "lxml")
                ul = soup.find('ul', id='subdomains')
                if not ul:
                    return []
                subdomains = ul.findAll("li")
                for result in subdomains:
                    subdomain=f"{result.text.strip()}.{domain}"
                    shodanxs.append(subdomain)
                return shodanxs
            
    except aiohttp.ServerConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for ShodanX, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for ShodanX, due to Serverside Error", file=sys.stderr)
                    
    except aiohttp.ClientConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for ShodanX, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for ShodanX, due to Clientside connection Error", file=sys.stderr) 
                    
    except KeyboardInterrupt as e:
        quit()
        
    except TimeoutError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for ShodanX , due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for ShodanX, due to Timeout Error", file=sys.stderr)
                           
    except Exception as e:
        if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in shodan request block: {e}, {type(e)}{reset}", file=sys.stderr)


import os
def get_username():
    try:
        username = os.getlogin()
    except OSError:
        username = os.getenv('USER') or os.getenv('LOGNAME') or os.getenv('USERNAME') or 'Unknown User'
    except Exception as e:
        username = "Unknown User"    
    return username


from colorama import Fore, Style
import aiofiles
import random
import yaml
import aiohttp
import sys
from fake_useragent import UserAgent

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def censys(domain, session, configs, username, args): # code converted from subfinder gloang --> Subdominator python
    
    try:
        
        try:
            
            async with aiofiles.open(configs, "r") as streamr:
                
                data = yaml.safe_load(await streamr.read())
                
                if "censys" not in data:
                    
                    return []
                
                rand = data.get("censys", [])
                
                if rand is None:
                    
                    return []
                randomkeys = random.choice(data.get("censys", []))
                
                if randomkeys is None:
                    
                    return []
                
                randomtoken, randomsecret = randomkeys.split(":")
                
                if randomtoken is None or randomsecret is None:
                    
                    return []
                
        except yaml.YAMLError as e:
            
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)  
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                               
        except TypeError as e:
            
            pass 
        
        except KeyboardInterrupt as e:
            quit()
        
        except yaml.YAMLObject as e:
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)    
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr)
                    
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in censys reader block: {e}, {type(e)}{reset}", file=sys.stderr)
                    
        try:
            
            censyss = []
            url = "https://search.censys.io/api/v2/certificates/search"
            maxpage = 10
            maxdata = 100
            cursor = None
            params = {'q': domain, 'per_page': maxdata}
            proxy = args.proxy if args.proxy else None
            
            for _ in range(maxpage+1):
                
                if cursor:
                    
                    params['cursor'] = cursor
                    
                async with session.get(url, auth=aiohttp.BasicAuth(randomtoken, randomsecret), params=params, proxy=proxy, ssl=False, timeout=args.timeout) as response:
                    
                    data = await response.json()
                    if response.status != 200:
                        if args.show_key_info:
                            if not args.no_color:
                                print(f"[{bold}{red}ALERT{reset}]: Censys blocking our request, {username} please check your api usage for this keys: {randomsecret}, {randomtoken}", file=sys.stderr)
                            else:
                                print(f"[ALERT]: Censys blocking our request, {username} please check your api usage for this keys: {randomsecret}, {randomtoken}", file=sys.stderr)
                        if len(censyss) > 0:
                            return censyss
                        else:
                            return []
                    
                    if 'result' in data and 'hits' in data['result']:
                        
                        for hit in data['result']['hits']:
                            
                            for name in hit.get('names', []):
                                
                                censyss.append(name)
                                
                        cursor = data['result']['links'].get('next')
                        
                        if not cursor:
                            
                            break
                        
            return censyss
        
        except aiohttp.ServerConnectionError as e:
            
            if args.show_timeout_info:
                 
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Censys API, due to Serverside Error", file=sys.stderr)    
                    
                else:
                    
                    print(f"[INFO]: Timeout reached for Censys API, due to Serverside Error", file=sys.stderr)
                
        except aiohttp.ClientConnectionError as e:
            
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Censys API, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Censys API, due to Clientside connection Error", file=sys.stderr) 
                    
        except TimeoutError as e:
            
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Censys API, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Censys API, due to Timeout Error", file=sys.stderr) 
                
        except KeyboardInterrupt as e:
            quit()
            
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api censys req block: {e}, due to: {type(e)}{reset}", file=sys.stderr)
                    
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api censys main block: {e}, due to: {type(e)}{reset}", file=sys.stderr)


import re
from colorama import Fore, Style # to reduce imports and colorsvar assingings 
import tldextract

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def compiler(domain):
    pattern = re.compile(r"(?i)[a-zA-Z0-9\*_.-]+\." + re.escape(domain))
    return pattern

async def extracts(response, domain):
    regex = await compiler(domain)
    matches = regex.findall(response)
    return set(matches)
    
def check_subdomain(domain):
    parsed = tldextract.extract(domain)
    return parsed


from colorama import Fore, Style
import aiofiles
import random
import yaml
import aiohttp
import sys
import base64

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def fofa(domain, session, configs, username, args):
    try:
        try:
            async with aiofiles.open(configs, "r") as streamr:
                
                data = yaml.safe_load(await streamr.read())
                
                if "fofa" not in data:
                    return 
                
                randvalue = data.get("fofa", [])
                
                if randvalue is None:  
                    return []
                   
            randomkey = random.choice(data.get("fofa", []))
            
        except yaml.YAMLError as e:
            
            if args.show_key_info: 
                
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)  
                      
                else:
                    
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                               
        except TypeError as e:
            
            pass 
        
        except yaml.YAMLObject as e:
            
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)    
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                    
        except KeyboardInterrupt as e:
            quit()
            
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in fofa reader block: {e}, {type(e)}{reset}",file=sys.stderr)
                
        if randomkey is None:
            return []
        
        try:
            
            FOFA = []
            pagenum = 1
            domain_encoded = f"""domain="{domain}" """.encode('utf-8')
            subdomains = base64.b64encode(domain_encoded).decode('utf-8')
            
            proxy = args.proxy if args.proxy else None
            while True:
                url = f"https://fofa.info/api/v1/search/all?key={randomkey}&qbase64={subdomains}&page={pagenum}&full=true&size=1000"
                async with session.get(url, timeout=args.timeout, proxy=proxy, ssl=False) as response:
                    if response.status != 200:
                        return FOFA
                    data = await response.json()
                    if "results" not in data:
                        return FOFA
                    for result in data.get('results', []):
                        url = result[0]
                        if url.startswith("https://"):
                            url = url.replace("https://", "")
                        elif url.startswith("http://"):
                            url = url.replace("http://", "") 
                        else :
                            url = url
                        if ':' in url:
                            subdomain = url.split(':')[0]
                        else:
                            subdomain = url
                        FOFA.append(subdomain)
                        
                        for r in FOFA:
                            print(r)
                    
                    size = data.get('size')
                    print(size)
                    if size < 1000:
                        return FOFA
                    pagenum += 1
    
        except aiohttp.ServerConnectionError as e:
            
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for FOFA API, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for FOFA API, due to Serverside Error", file=sys.stderr)
                
        except aiohttp.ClientConnectionError as e:
            
            if args.show_timeout_info:
                 
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for FOFA API, due to ClientSide connection Error", file=sys.stderr)    
                    
                else:
                    
                    print(f"[INFO]: Timeout reached for FOFA API, due to Clientside connection Error", file=sys.stderr) 
        
        except KeyboardInterrupt as e:
            quit()
                           
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api fofa req block: {e}, due to: {type(e)}{reset}", file=sys.stderr)
            
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api fofa main block: {e}, due to: {type(e)}{reset}", file=sys.stderr)
        


from colorama import Fore, Style
import aiofiles
import random
import yaml
import aiohttp
import sys
import asyncio

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def subdomaincenter(domain, session, args):
    try:
        subdomaincenters = []
        url = f"https://api.subdomain.center/?domain={domain}"
        proxy = args.proxy if args.proxy else None
        async with session.get(url, timeout=args.timeout, proxy=proxy, ssl=False) as response:
            if response.status !=200:
                return
            data = await response.json()
            for subdomain in data:
                subdomaincenters.append(subdomain)
            return subdomaincenters
    except aiohttp.ServerConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Subdomaincenter API, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Subdomaincenter API, due to Serverside Error", file=sys.stderr)
    except aiohttp.ClientConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Subdomaincenter API, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Subdomaincenter API, due to Clientside connection Error", file=sys.stderr)
    except TimeoutError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Subdomaincenter, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Subdomaincenter, due to Timeout Error", file=sys.stderr)
                           
    except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in subdomaincenter request block: {e}, {type(e)}{reset}", file=sys.stderr)


from colorama import Fore, Style
import aiofiles
import random
import yaml
import aiohttp
import sys

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL


async def facebook(domain, session, configs, username, args):
    try:
        
        try:
            async with aiofiles.open(configs, "r") as streamr:
                data = yaml.safe_load(await streamr.read())
                if "facebook" not in data:
                    return []
                randvalue = data.get("facebook", [])
                if randvalue is None:  
                    return []
                
            randomkeys = random.choice(data.get("facebook", []))
            randomid, randomsecret = randomkeys.split(":")
            
            if randomid is None or randomsecret is None:
                return []
        
        except yaml.YAMLError as e:
            
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)  
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                               
        except TypeError as e:
            pass 
        
        except KeyboardInterrupt as e:
            quit()
        
        except yaml.YAMLObject as e:
            
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)    
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr)
            
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception at fb key loader: {e}, {type(e)}{reset}", file=sys.stderr)
            
        try:
            fbcerts = []
            randomtoken = f"{randomid}|{randomsecret}"
            url = f"https://graph.facebook.com/v18.0/certificates?fields=domains&access_token={randomtoken}&query={domain}&limit=1000"
            proxy = args.proxy if args.proxy else None
            while True:
                async with session.get(url, timeout=args.timeout, proxy=proxy, ssl=False) as response:
                    if response.status != 200:
                        return fbcerts
                    data = await response.json()
                    for item in data['data']:
                        subdomains = item['domains']
                        for subdomain in subdomains:
                            if subdomain.endswith(f"{domain}"):
                                fbcerts.append(subdomain)
                    pages = data.get("paging", {})
                    next_page = pages.get('next')
                    if next_page:
                        url = next_page
                    if not next_page:
                        return fbcerts

        except aiohttp.ServerConnectionError as e:
            
            if args.show_timeout_info:
                 
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Facebook API, due to Serverside Error", file=sys.stderr)    
                    
                else:
                    
                    print(f"[INFO]: Timeout reached for Facebook API, due to Serverside Error", file=sys.stderr)
                
        except aiohttp.ClientConnectionError as e:
            
            if args.show_timeout_info:
                 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Facebook API, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Facebook API, due to Clientside connection Error", file=sys.stderr) 
        
        except KeyboardInterrupt as e:
            quit()
            
        except TimeoutError as e:
           if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Facebook API, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Facebook API, due to Timeout Error", file=sys.stderr) 
           return fbcerts
                           
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api facebook req block: {e}, due to: {type(e)}{reset}", file=sys.stderr)
            
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception at fb main: {e}, {type(e)}{reset}", file=sys.stderr)
            
        

        


import asyncio
import httpx
import aiofiles
import os  
import random
from fake_useragent import UserAgent
from datetime import datetime
from colorama import Fore, Style
import re
from subdominator.modules.utils import compiler, extracts
red =  Fore.RED
green = Fore.GREEN
magenta = Fore.MAGENTA
cyan = Fore.CYAN
mixed = Fore.RED + Fore.BLUE
blue = Fore.BLUE
yellow = Fore.YELLOW
white = Fore.WHITE
reset = Style.RESET_ALL
bold = Style.BRIGHT
colors = [ green, cyan, blue]
random_color = random.choice(colors)

Commoncrawls = set()

async def indexDB(args):
    try:
        headers = {
                "User-Agent": UserAgent().random
        }
        async with httpx.AsyncClient(headers=headers, timeout=args.timeout, verify=False,proxy=args.proxy if args.proxy else None) as session:
            response = await session.request("GET", "https://index.commoncrawl.org/collinfo.json", follow_redirects=True)
            return response.json()
    except KeyboardInterrupt:
        exit()
    except asyncio.CancelledError:
        exit()
    except httpx.RemoteProtocolError:
        pass
    except httpx.ReadTimeout:
        pass
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception at CC index fetcher: {e}, {type(e)}{reset}")
            
async def CcClient(args,searchurl: str,domain):
    try:
        headers = {
                "User-Agent": UserAgent().random
        }
        async with httpx.AsyncClient(timeout=httpx.Timeout(read=300.0, connect=args.timeout, write=None, pool=None), headers=headers, verify=False, proxy=args.proxy if args.proxy else None) as request:
            async with  request.stream("GET", f"{searchurl}?url=*.{domain}", follow_redirects=True) as response:
                async for url in response.aiter_lines():

                    subdomains = await extracts(url, domain)
                    if subdomains:
                        for subdomain in subdomains:
                            subdomain = subdomain.lstrip("25").lstrip("2F").lstrip("40").lstrip(".")
                            if subdomain not in Commoncrawls and not subdomain.startswith("%3D") and not subdomain.startswith("3D"):
                                Commoncrawls.add(subdomain)
    except KeyboardInterrupt:
            exit()
    except asyncio.CancelledError as e:
            exit()
    except httpx.RemoteProtocolError:
            pass
    except httpx.ReadTimeout:
            pass
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception at  CC url fetcher: {e}, {type(e)}{reset}")

async def commoncrawl(Domain, args):
    try:
        indexurls = []
        added = set()
        responsed = await indexDB(args)
        if responsed is None:
            return Commoncrawls
        ctyear = datetime.now().year
        years = [str(ctyear - i) for i in range(6)]
        for year in years:
            for index in responsed:
                if year not in added:
                    if year in index.get("name"):
                        indexurls.append(index.get('cdx-api'))
                        added.add(year)
        for url in indexurls:
            await CcClient(args, url, Domain)
        return Commoncrawls
    except KeyboardInterrupt:
            exit()
    except asyncio.CancelledError as e:
            exit()
    except Exception as e:
            if args.sec_deb:
                    print(f"Exception at commoncrawl: {e}, {type(e)}")


from colorama import Fore, Style
import aiofiles
import random
import yaml
import aiohttp
import sys
from fake_useragent import UserAgent

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def binaryget(domain, session, randkey, pagenum, pagesize, args):
    
    try:
        url = f"https://api.binaryedge.io/v2/query/domains/subdomain/{domain}?page={pagenum}&pagesize={pagesize}"
        auth = {
            'User-Agent': UserAgent().random,
            'X-Key': f'{randkey}'
        }
        proxy = args.proxy if args.proxy else None
        async with session.get(url, timeout=args.timeout, headers=auth, proxy=proxy, ssl=False) as response:
            data = await response.json()
            if response.status != 200:
                return 
            subdomains = data.get("events", [])
            if subdomains:
                return subdomains
            
    except aiohttp.ServerConnectionError as e:
            
        if args.show_timeout_info:
            if not args.no_color: 
                print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Binaryegde API, due to Serverside Error", file=sys.stderr)    
            else:
                print(f"[INFO]: Timeout reached for Binaryegde API, due to Serverside Error", file=sys.stderr)
                    
    except TimeoutError as e:
        if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Binaryegde API, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Binaryegde API, due to Timeout Error", file=sys.stderr)
                    
    except KeyboardInterrupt as e:
        quit()
                
    except aiohttp.ClientConnectionError as e:
            
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Binaryegde API, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Binaryegde API, due to Clientside connection Error", file=sys.stderr) 
            
    except Exception as e:
        if args.sec_deb:
            print(f"Exception in binaryget: {e}, {type(e)}",file=sys.stderr)
        

async def binaryedge(domain, session, configs, username, args):
        
    try:
        
        try:
            
            binaryedges = []
            
            async with aiofiles.open(configs, "r") as streamr:
                
                data = yaml.safe_load(await streamr.read())
                
                if "binaryedge" not in data:
                    
                    return []
                
                randvalue = data.get("binaryedge", [])
                
                if randvalue is None:  

                    return []

            randomkey = random.choice(data.get("binaryedge", []))
            
            if randomkey is None:
                
                return []
            pagenum = 1
            pagesize = 100
        
            while True:
            
                subdomains = await binaryget(domain, session, randomkey, pagenum, pagesize, args)
            
                if subdomains:
                
                    for subdomain in subdomains:
                    
                        binaryedges.append(subdomain)
                    
                if not subdomains:
                    if len(binaryedge)>0:
                        return binaryedges
                    else:
                        return []
            
                pagenum += 1
            
        except yaml.YAMLError as e:
            
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)  
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                               
        except TypeError as e:
            pass 
        
        except yaml.YAMLObject as e:
            
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)    
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                    
    except Exception as e:
        if args.sec_deb:
            print(f"Exception occured in api binaryedge reader block: {e}, due to: {type(e)}", file=sys.stderr)
        


from colorama import Fore, Style
import aiofiles
import random
import yaml
import aiohttp
import sys
import asyncio

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def shodan(domain, session, configs, username, args):
    
    try:
        
        try:
            
            async with aiofiles.open(configs, "r") as streamr:
                
                data = yaml.safe_load(await streamr.read())
            
            if "shodan" not in data:
                return
            
            rand = data.get("shodan", [])
            
            if rand is None:
                return
            
            randomkey = random.choice(data.get("shodan", []))
            
            if randomkey is None:
                return
        
        except yaml.YAMLError as e:
            
            if args.show_key_info: 
                
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)  
                      
                else:
                    
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                               
        except TypeError as e:
            
            pass 
        
        except yaml.YAMLObject as e:
            
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)    
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr)
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in shodan reader block: {e}, {type(e)}{reset}", file=sys.stderr)
                
        try:
            
            shodans = []
            
            url = f"https://api.shodan.io/dns/domain/{domain}?key={randomkey}"
            proxy = args.proxy if args.proxy else None
            
            async with session.get(url, timeout=args.timeout, proxy=proxy, ssl=False) as response:
                data = await response.json()
                if response.status != 200:
                    return
                
                subdomains = data.get("subdomains", [])
                
                for subdomain in subdomains:
                    
                    shodans.append(f"{subdomain}.{domain}")
                    
                return shodans
            
        except aiohttp.ServerConnectionError as e:
            
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Shodan API, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Shodan API, due to Serverside Error", file=sys.stderr)
                
        except aiohttp.ClientConnectionError as e:
            
            if args.show_timeout_info:
                 
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Shodan API, due to ClientSide connection Error", file=sys.stderr)    
                    
                else:
                    
                    print(f"[INFO]: Timeout reached for Shodan API, due to Clientside connection Error", file=sys.stderr) 
        
        except KeyboardInterrupt as e:
            quit()
        
        except TimeoutError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Shodan API, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Shodan API, due to Timeout Error", file=sys.stderr)
                           
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in shodan request block: {e}, {type(e)}{reset}", file=sys.stderr)
            
    except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in shodan main block: {e}, {type(e)}{reset}", file=sys.stderr)
        


from colorama import Fore, Style
import aiohttp
import sys
import re

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def sitedossier(domain, session, args):
    try:
        sitedossiers=[]
        page = 1
        while True:
            url = f"http://www.sitedossier.com/parentdomain/{domain}/{page}"
            proxy = args.proxy if args.proxy else None
            async with session.get(url, timeout=args.timeout, proxy=proxy, ssl=False) as response:
                if response.status !=200:
                    if len(sitedossiers)>0:
                        return sitedossiers
                    return []
                data = await response.text()
                filterdomain = re.escape(domain)
                pattern = r'(?i)(?:https?://)?([a-zA-Z0-9*_.-]+\.' + filterdomain + r')'
                subdomains = re.findall(pattern, data)
                for subdomain in subdomains:
                    sitedossiers.append(subdomain)
                if "Show next 100 items" not in data:
                    if len(sitedossiers)>0:
                        return sitedossiers
                    return []
                page+=100
    except aiohttp.ServerConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Sitedossier, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Sitedossier, due to Serverside Error", file=sys.stderr)
    except aiohttp.ClientConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Sitedossier, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Sitedossier, due to Clientside connection Error", file=sys.stderr)
                    
    except KeyboardInterrupt as e:
        quit()
        
    except TimeoutError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Sitedossier, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Sitedossier, due to Timeout Error", file=sys.stderr)
                           
    except Exception as e:
        if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in sitedossier request block: {e}, {type(e)}{reset}", file=sys.stderr)


from colorama import Fore, Style
import aiofiles
import random
import yaml
import httpx
import sys
import os
import asyncio
import json
bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

merklemaps = []
async def Merklemap(Domain, args): # use this method
    try:
        page = 1
        proxy = args.proxy if args.proxy else None
        while True:
            async with httpx.AsyncClient(proxy=proxy, verify=False) as session:
                response = await session.request("GET", f"https://api.merklemap.com/search?query=*.{Domain}&page={page}", timeout=httpx.Timeout(connect=args.timeout, read=1000.0, write=None, pool=None))
                data = response.json()
                if not data['results']:
                    return merklemaps
                for result in data['results']:
                    if 'domain' in result:
                        domain = result.get('domain')
                        cname = result.get('subject_common_name')
                        merklemaps.append(domain)
                        merklemaps.append(cname)
                page +=1
    except httpx._exceptions.ConnectTimeout:
        if args.show_timeout_info:
            if not args.no_color:   
                print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Merklemap API, due to ClientSide connection Error", file=sys.stderr)      
            else:  
                print(f"[INFO]: Timeout reached for Merklemap API, due to Clientside connection Error", file=sys.stderr) 
    except httpx._exceptions.ConnectError:
        if args.show_timeout_info:
            if not args.no_color:   
                print(f"[{bold}{red}INFO{reset}]: {bold}{white}Unable to establish client connection to Merklemap API{reset}", file=sys.stderr)      
            else:  
                print(f"[INFO]: Unable to establish client connection to Merklemap API", file=sys.stderr) 
    except httpx._exceptions.ReadTimeout:
        return merklemaps
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]Exception occred in merklemap: {e}, {type(e)}", file=sys.stderr)
        return merklemaps
        
async def merklemap(Domain, args):
    try:
        resploader = []
        url = f"https://api.merklemap.com/search?query=*.{Domain}&stream=true"
        async with httpx.AsyncClient() as client:
            async with client.stream("GET", url, timeout=httpx.Timeout(connect=30, read=1000.0, write=None, pool=None)) as response:
                lines = []
                async for line in response.aiter_lines():
                    if line.startswith("data: "):
                        resploader.append(line[6:])
        for line in resploader:
            json_data = json.loads(line)
            domain = json_data.get("domain",'')
            cname  = json_data.get("subject_common_name",'')
            merklemaps.append(domain)
            merklemaps.append(cname)
    except httpx._exceptions.ConnectTimeout:
        if args.show_timeout_info:
            if not args.no_color:   
                print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Merklemap API, due to ClientSide connection Error", file=sys.stderr)      
            else:  
                print(f"[INFO]: Timeout reached for Merklemap API, due to Clientside connection Error", file=sys.stderr) 
    except httpx._exceptions.ConnectError:
        if args.show_timeout_info:
            if not args.no_color:   
                print(f"[{bold}{red}INFO{reset}]: {bold}{white}Unable to establish client connection to Merklemap API{reset}", file=sys.stderr)      
            else:  
                print(f"[INFO]: Unable to establish client connection to Merklemap API", file=sys.stderr) 
    except httpx._exceptions.ReadTimeout:
        return merklemaps
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]Exception occred in merklemap: {e}, {type(e)}", file=sys.stderr)
    finally:
        return merklemaps
    


#!/usr/bin/env python3
from colorama import Fore,Back,Style
import argparse
import time as t
import random
red =  Fore.RED
green = Fore.GREEN
magenta = Fore.MAGENTA
cyan = Fore.CYAN
mixed = Fore.RED + Fore.BLUE
blue = Fore.BLUE
yellow = Fore.YELLOW
white = Fore.WHITE
reset = Style.RESET_ALL
bold = Style.BRIGHT
colors = [ green, cyan, blue]
random_color = random.choice(colors)
def cli():
    try:
        parser = argparse.ArgumentParser(add_help=False, usage=argparse.SUPPRESS,exit_on_error=False)
        parser.add_argument("-d", "--domain", type=str)
        parser.add_argument("-dL", "--domain-list", type=str)
        parser.add_argument("-cp", "--config-path", type=str)
        parser.add_argument("-o", "--output", type=str)
        parser.add_argument("-oD", "--output-directory", type=str)
        parser.add_argument("-t", "--timeout", type=float, default=30)
        parser.add_argument("-up", "--update", action="store_true")
        parser.add_argument("-nt", "--notify", action="store_true")
        parser.add_argument("-px", "--proxy", type=str)
        parser.add_argument("-h", "--help", action="store_true")
        parser.add_argument("-v","--version", action="store_true")
        parser.add_argument("-ski", "--show-key-info", action="store_true")
        parser.add_argument("-ste", "--show-timeout-info", action="store_true")
        parser.add_argument("-nc", "--no-color", action="store_true")
        parser.add_argument("-ls", "--list-source", action="store_true")
        parser.add_argument("-duc", "--disable-update-check", action="store_true")
        parser.add_argument("-sd", "--sec-deb", action="store_true")
        parser.add_argument("-sup", "--show-updates", action="store_true")     
        parser.add_argument("-fw", "--filter-wildcards", action="store_true")
        parser.add_argument("-oJ", "--output-json", type=str)
        parser.add_argument("-s", "--silent", action="store_true")
        global args 
                
        return parser.parse_args()
        
    
    except argparse.ArgumentError as e:
        
        print(f"[{bold}{blue}INFO{reset}]: {bold}{white}Please use the command for more infromation:{reset} {bold}{blue}subdominator -h{reset}")
        
        exit()
        
    except argparse.ArgumentTypeError as e:
        
        print(f"[{bold}{blue}INFO{reset}]: {bold}{white}Please use the command for more infromation:{reset} {bold}{blue}subdominator -h{reset}")
        
        exit()
        
        
    except Exception as e:
        
        pass
    
    except KeyboardInterrupt as e:
        
        print(f"\n[{bold}{blue}INF{reset}]: {bold}{white}CTRL+C Pressed{reset}")
        
        SystemExit


from colorama import Fore, Style
import aiohttp
import sys
from fake_useragent import UserAgent

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def columbusapi(domain, session, args):
    try:
        columbusapis = []
        url = f"https://columbus.elmasy.com/api/lookup/{domain}?days=-1"
        proxy= args.proxy if args.proxy else None
        headers = {
            "User-Agent": UserAgent().random
        }
        
        async with session.get(url, timeout=args.timeout, proxy=proxy, ssl=False, headers=headers) as response:
            if response.status !=200:
                return []
            data=await response.json()
            for subdomain in data:
                columbusapis.append(f"{subdomain}.{domain}")
            return columbusapis
        
    except aiohttp.ServerConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Columbus API, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Columbus API, due to Serverside Error", file=sys.stderr)
    except TimeoutError as e:
        if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Columbus API, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Columbus API, due to Timeout Error", file=sys.stderr) 
    except aiohttp.ClientConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Columbus API, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Columbus API, due to Clientside connection Error", file=sys.stderr) 
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured at columbusapi: {e}, {type(e)}{reset}", file=sys.stderr)


from colorama import Fore, Style
import aiohttp
import sys

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def urlscan(domain , session, args):
    try:
        urlscans = []
        url = f"https://urlscan.io/api/v1/search/?q=page.domain:{domain}&size=10000"
        proxy= args.proxy if args.proxy else None
        async with session.get(url, timeout=args.timeout, proxy=proxy,ssl=False) as response:
            if response.status != 200:
                return
            data = await response.json()
            for domain in data['results']:
                subdomain = domain['page']['domain']
                urlscans.append(subdomain)
            return urlscans
    except aiohttp.ServerConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Urlscan, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Urlscan due to Serverside Error", file=sys.stderr)
    except aiohttp.ClientConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Urlscan, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Urlscan, due to Clientside connection Error", file=sys.stderr) 
                    
    except KeyboardInterrupt as e:
        quit()
        
    except TimeoutError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Urlscan, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Urlscan, due to Timeout Error", file=sys.stderr)
                           
    except Exception as e:
        if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in urlscan request block: {e}, {type(e)}{reset}", file=sys.stderr)
                


import requests
from colorama import Fore, Style
import subprocess
import os
from rich.console import Console
from rich.markdown import Markdown

console = Console()
bold =Style.BRIGHT
blue = Fore.BLUE
red  = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

def getzip(): 
    try:
        url = "https://api.github.com/repos/RevoltSecurities/Subdominator/releases/latest"
        response = requests.get(url, timeout=10)
        if response.status_code == 200:
            return response.json()['zipball_url']
        else:
            print(f"[{bold}{red}ALERT{reset}]: {bold}{white}Hey Update Failed for Subdominator, Please try to update the Subdominator manually{reset}")
            quit()
    except Exception as e:
        pass
    

def launch(url,config):
    try:
        response = requests.get(url, timeout=20, stream=True)
        filepath = f"{config}/Subdominator.zip"
        if response.status_code == 200:
            print(f"[{bold}{blue}INFO{reset}]: {bold}{white}Downloading Subdominator latest version...{reset}")
            with open(f"{filepath}", "wb") as streamw:
                for data in response.iter_content():
                    if data:
                        streamw.write(data)
            try:
                subprocess.run(["pip", "install", f"{filepath}"], check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
            except Exception as e:
                print(f"[{bold}{red}ALERT{reset}]: {bold}{white}Update Failed for Subdominator, Please try to update the Subdominator manually{reset}")
                os.remove(filepath)
                quit()
            finally:
                os.remove(filepath)
        else:
            print(f"[{bold}{red}ALERT{reset}]: {bold}{white}Update Failed for Subdominator, Please try to update the Subdominator manually{reset}")
            quit()
    except Exception as e:
        pass

def updatelog():
    try:
        
        url = f"https://raw.githubusercontent.com/RevoltSecurities/Subdominator/main/subdominator/updatelog.MD"
        
        response = requests.get(url, timeout=20, stream=True)
        
        if response.status_code == 200:
            
            loader = response.text
            
            console.print(Markdown(loader))
            
        else:
            print(f"[{bold}{red}ALERT{reset}]: {bold}{white}Hey  unable to fetch update logs so please visit here --> https://github.com/RevoltSecurities/Subdominator{reset}")
            quit()
    except Exception as e:
        pass
    


from colorama import Fore, Style
import aiohttp
import sys
from fake_useragent import UserAgent

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def myssl(domain, session, args):
    try:
        myssls = []
        
        url= f"https://myssl.com/api/v1/discover_sub_domain?domain={domain}"
        headers = {
            "User-Agent": UserAgent().random
        }
        proxy = args.proxy if args.proxy else None
        async with session.get(url, headers=headers, timeout=args.timeout, ssl=False, proxy=proxy) as response:
            if response.status !=200:
                return []
            data = await response.json()
            for subdomains in data.get("data"):
                subdomain = subdomains['domain']
                if subdomain.endswith(f".{domain}"):
                    myssls.append(subdomain)
            return myssls
    except aiohttp.ServerConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Myssl API, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Myssl API, due to Serverside Error", file=sys.stderr)
    except TimeoutError as e:
        if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Myssl API, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Myssl API, due to Timeout Error", file=sys.stderr) 
    except aiohttp.ClientConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Myssl API, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Myssl API, due to Clientside connection Error", file=sys.stderr) 
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured at Myssl api: {e}, {type(e)}{reset}", file=sys.stderr)


from colorama import Fore, Style
import aiohttp
import sys
from fake_useragent import UserAgent
bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def cyfare(domain, session, args):
    try:
        Cyfare = []
        url = "https://cyfare.net/apps/subfind/query.php"
        headers = {
            "User-Agent": UserAgent().random
        }
        jbody = {
            "domain": f"{domain}"
        }
        proxy = args.proxy if args.proxy else None
        async with session.post(url, headers=headers, json=jbody, timeout=args.timeout, ssl=False, proxy=proxy) as response:
            if response.status != 200:
                return Cyfare
            data = await response.json()
            subdomains = data.get("subdomains", [])
            if len(subdomains) > 0:
                Cyfare.extend(subdomains)
            return Cyfare
    except aiohttp.ServerConnectionError as e:
                if args.show_timeout_info:
                    if not args.no_color: 
                        print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Cyfare, due to Serverside Error", file=sys.stderr)    
                    else:
                        print(f"[INFO]: Timeout reached for Cyfare, due to Serverside Error", file=sys.stderr)  
    except KeyboardInterrupt as e:
            quit()
    except TimeoutError as e:
        if args.show_timeout_info:
            if not args.no_color: 
                print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Cyfare, due to Timeout Error", file=sys.stderr)    
            else:
                print(f"[INFO]: Timeout reached for Cyfare, due to Timeout Error", file=sys.stderr)
    except aiohttp.ClientConnectionError as e:
        if args.show_timeout_info:
            if not args.no_color: 
                print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Cyfare, due to ClientSide connection Error", file=sys.stderr)    
            else:
                print(f"[INFO]: Timeout reached for Cyfare, due to Clientside connection Error", file=sys.stderr)
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception at cyfare API : {e}, {type(e)}{reset}", file=sys.stderr)


from colorama import Fore, Style
import asyncio
import aiohttp
import sys

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def shrewdeye(domain, session, args):
    try:
        shrewdeyes=[]
        url= f"https://shrewdeye.app/domains/{domain}.txt"
        proxy= args.proxy if args.proxy else None
        async with session.get(url, timeout=args.timeout, proxy=proxy, ssl=False) as response:
            if response.status !=200:
                return []
            data = await response.text()
            if len(data)<0:
                return []
            subdomains = data.split("\n")
            for subdomain in subdomains:
                shrewdeyes.append(subdomain)
            return shrewdeyes
    except aiohttp.ServerConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Shrewdeye, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Shrewdeye, due to Serverside Error", file=sys.stderr)
                
    except aiohttp.ClientConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Shrewdeye, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Shrewdeye, due to Clientside connection Error", file=sys.stderr)
                    
    except KeyboardInterrupt as e:
        quit()
        
    except TimeoutError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Shrewdeye, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Shrewdeye, due to Timeout Error", file=sys.stderr)
                           
    except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in shrewdeye request block: {e}, {type(e)}{reset}", file=sys.stderr)
    
            
            


from colorama import Fore, Style
import aiofiles
import random
import yaml
import aiohttp
import sys
from fake_useragent import UserAgent

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def getID(domain, session, host, key, args):
    try:
        baseurl = f"https://{host}/phonebook/search?k={key}"
        auth = {
            "User-Agent": UserAgent().random
        }
        
        reqbody = {
            "Maxresults": 100000,
            "Media": 0,
            "Target": 1,
            "Term": domain,
            "Terminate": None,
            "Timeout" : 20,
            
        }
        
        async with session.post(baseurl, headers=auth, timeout=10, json=reqbody) as response:
            if response.status != 200:
                return
            data = await response.json()
            id = data.get("id")
            return id
        
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured at intelx getID block: {e}, {type(e)}{reset}", file=sys.stderr)

async def intelx(domain , session, configs, username, args):
    
    try:
        
        try:
            intelxs = []
            async with aiofiles.open(configs, "r") as streamr:
                
                data = yaml.safe_load(await streamr.read())
                
                if "intelx" not in data:
                    
                    return 
                
                randvalue = data.get("intelx", [])
                
                if randvalue is None:  
                      
                    return  
                   
            randomkeys = random.choice(data.get("intelx", []))
            
            randhost, randkey = randomkeys.split(":")
            
            if randhost is None or randkey is None:
                return
            
        except yaml.YAMLError as e:
            
            if args.show_key_info: 
                
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)  
                      
                else:
                    
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                               
        except TypeError as e:
            
            pass 
        
        except yaml.YAMLObject as e:
            
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)    
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                    
        except KeyboardInterrupt as e:
            quit()
            
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in intelx reader block: {e}, {type(e)}{reset}", file=sys.stderr)
            
        try:
            
            id = await getID(domain, session, randhost, randkey, args)
            if not id:
                return
            
            while True:
                baseurl = f"https://{randhost}/phonebook/search/result?k={randkey}&id={id}&limit=10000"
                auth = {
                "User-Agent": UserAgent().random
                }
                proxy = args.proxy if args.proxy else None
                async with session.get(baseurl, headers=auth, timeout=args.timeout, proxy=proxy, ssl=False) as response:
                    if response.status != 200:
                        return
                    data = await response.json()
                    for item in data.get('selectors', []):
                        subdomain = item.get('selectorvalue')
                        if subdomain:
                            intelxs.append(subdomain)
                        
                    if data.get('status') not in [0, 3]:
                        return intelxs
                    
        except aiohttp.ServerConnectionError as e:
            
            if args.show_timeout_info:
                 
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for IntelX API, due to Serverside Error", file=sys.stderr)    
                    
                else:
                    
                    print(f"[INFO]: Timeout reached for IntelX API, due to Serverside Error", file=sys.stderr)
                    
                
        except aiohttp.ClientConnectionError as e:
            
            if args.show_timeout_info:
                 
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for IntelX API, due to ClientSide connection Error", file=sys.stderr)    
                    
                else:
                    
                    print(f"[INFO]: Timeout reached for IntelX API, due to Clientside connection Error", file=sys.stderr) 
        
        except TimeoutError as e:
            if args.show_timeout_info:
                 
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for IntelX API, due to Timeout Error", file=sys.stderr)    
                    
                else:
                    
                    print(f"[INFO]: Timeout reached for IntelX API, due to Timeout Error", file=sys.stderr)
        
        except KeyboardInterrupt as e:
            quit()

        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occred at intelx req block: {e}, {type(e)}{reset}", file=sys.stderr)
        
    except Exception as e:
        
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception at intelx main block: {e}, {type(e)}{reset}", file=sys.stderr)


from colorama import Fore, Style
import aiofiles
import random
import yaml
import aiohttp
import sys

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def redhuntapi(url, domain, session, randkey, pagenum, pagesize, args):
    try:
        
        base_url = f"{url}?domain={domain}&page_size={pagesize}&page={pagenum}"
        auth = {
            "X-BLOBR-KEY": randkey
        }
        proxy = args.proxy if args.proxy else None
        async with session.get(base_url, headers=auth, timeout=args.timeout, proxy=proxy, ssl=False) as response:
            
            if response.status !=200:
                return 
            data = await response.json()
            
            if 'subdomains' in data:
            
                return data['subdomains']
            return 
    except TimeoutError as e:
        if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Redhuntlabs API, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Redhuntlabs API, due to Timeout Error", file=sys.stderr)
                    
    except aiohttp.ServerConnectionError as e:
            
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Redhuntlabs API, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Redhuntlabs API, due to Serverside Error", file=sys.stderr)
                
    except aiohttp.ClientConnectionError as e:
            
            if args.show_timeout_info:
                 
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Redhuntlabs API, due to ClientSide connection Error", file=sys.stderr)    
                    
                else:
                    
                    print(f"[INFO]: Timeout reached for Redhuntlabs API, due to Clientside connection Error", file=sys.stderr) 
            
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception at request redhuntlabs: {e}, {type(e)}{reset}", file=sys.stderr)
            
    
async def redhuntlabs(domain, session, configs, username, args):
    
    try:
        try:
            
            redhuntlabss = []
            
            async with aiofiles.open(configs, "r") as streamr:
                
                data = yaml.safe_load(await streamr.read())
                
                if "redhuntlabs" not in data:
                    
                    return
                
                rand = data.get("redhuntlabs", [])
                
                if rand is None:
                    
                    return
                randomkeys = random.choice(data.get("redhuntlabs", []))
                
                if randomkeys is None:
                    
                    return
                url, randkeys = randomkeys.rsplit(":", 1)
                
                if randkeys is None:
                    return
                
                if url:
                    url = url
                else:
                    url = f"https://reconapi.redhuntlabs.com/community/v1/domains/subdomains"
            
            pagenum = 1
            
            pagesize = 1000
            
            while True:
                
                subdomains = await redhuntapi(url, domain, session, randkeys, pagenum, pagesize, args)
                
                if not subdomains:
                    if len(redhuntlabss)>0:
                        return redhuntlabss
                    return 
                
                redhuntlabss.extend(subdomains)
                
                pagenum +=1
                
        except yaml.YAMLError as e:
            
            if args.show_key_info: 
                
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)  
                      
                else:
                    
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                               
        except TypeError as e:
            
            pass 
        
        except yaml.YAMLObject as e:
            
            if args.show_key_info: 
                
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)    
                    
                else:
                    
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr)
            
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception at main redhuntlabs: {e}, {type(e)}{reset}", file=sys.stderr)
                    
                
                
            
            
            


import os
import sys
import random
from colorama import Fore, Style
import time
import asyncio
from rich.console import Console
from rich.markdown import Markdown
import aiohttp

red =  Fore.RED
green = Fore.GREEN
magenta = Fore.MAGENTA
cyan = Fore.CYAN
mixed = Fore.RED + Fore.BLUE
blue = Fore.BLUE
yellow = Fore.YELLOW
white = Fore.WHITE
reset = Style.RESET_ALL
bold = Style.BRIGHT
colors = [ green, cyan, blue]
random_color = random.choice(colors)

try:
    from .banner.banner import banner
    from .cli.cli import cli
    from .config.config import config, custompath, cachedir
    from .help.help import help
    from .source.source import sources
    from .update.update import *
    from .verify.verify import getverify
    from .version.version import version
    from .enumerates.abuseipdb.abuseipdb import abuseipdb
    from .enumerates.alienvault.alienvault import alienvault
    from .enumerates.anubis.anubis import anubis
    from .enumerates.bevigil.bevigil import bevigil
    from .enumerates.binaryedge.binaryedge import binaryedge
    from .enumerates.bufferover.bufferover import bufferover
    from .enumerates.builtwith.builtwith import builtwith
    from .enumerates.c99.c99 import c99
    from .enumerates.censys.censys import censys
    from .enumerates.certspotter.certspotter import certspotter
    from .enumerates.chaos.chaos import chaos
    from .enumerates.coderog.coderog import coderog
    from .enumerates.columbusapi.columbusapi import columbusapi
    from .enumerates.commoncrawl.commoncrawl import commoncrawl
    from .enumerates.crtsh.crtsh import crtsh
    from .enumerates.cyfare.cyfare import cyfare
    from .enumerates.digitorus.digitorus import digitorus
    from .enumerates.dnsdumpster.dnsdumpster import dnsdumpster
    from .enumerates.dnsrepo.dnsrepo import dnsrepo
    from .enumerates.facebook.facebook import facebook
    from .enumerates.fullhunt.fullhunt import fullhunt
    from .enumerates.fofa.fofa import fofa
    from .enumerates.google.google import google
    from .enumerates.hackertarget.hackertarget import hackertarget
    from .enumerates.huntermap.huntermap import huntermap
    from .enumerates.intelx.intelx import intelx
    from .enumerates.leakix.leakix import leakix
    from .enumerates.merkelmap.merklemap import merklemap
    from .enumerates.myssl.myssl import myssl
    from .enumerates.netlas.netlas import netlas
    from .enumerates.passivetotal.passivetotal import passivetotal
    from .enumerates.quake.quake import quake
    from .enumerates.racent.racent import racent
    from .enumerates.rapidapi.rapidapi import rapidapi
    from .enumerates.rapidfinder.rapidfinder import rapidfinder
    from .enumerates.rapidscan.rapidscan import rapidscan
    from .enumerates.rapiddns.rapiddns import rapiddns
    from .enumerates.redhuntlabs.redhuntlabs import redhuntlabs
    from .enumerates.rsecloud.rsecloud import rsecloud
    from .enumerates.securitytrails.securitytrails import securitytrails
    from .enumerates.shodan.shodan import shodan
    from .enumerates.shodanx.shodanx import shodanx
    from .enumerates.shrewdeye.shrewdeye import shrewdeye
    from .enumerates.sitedossier.sitedossier import sitedossier
    from .enumerates.subdomaincenter.subdomaincenter import subdomaincenter
    from .enumerates.trickest.trickest import trickest
    from .enumerates.urlscan.urlscan import urlscan
    from .enumerates.virustotal.virustotal import virustotal
    from .enumerates.waybackarchive.waybackarchive import waybackarchive
    from .enumerates.whoisxml.whoisxml import whoisxml
    from .enumerates.zoomeyeapi.zoomeyeapi import zoomeyeapi
    from .username.username import get_username
    from .extract.extract import  filters
    from .save.save import file, dir, jsonsave
    from .file.file import reader
    from .notify.notify import notify
except ImportError as e:
    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Import Error occured in  Subdominator Module imports due to: {e}{reset}", file=sys.stderr)
    print(f"[{bold}{blue}INFO{reset}]: {bold}{white}If you are encountering this issue more than a time please report the issues in Subdominator Github page.. {reset}", file=sys.stderr)
    exit()
    
args = cli()
if not args.config_path:
    configpath = config()
else:
    
    configpath = custompath(args)
    if not configpath:
        quit()
    
banners = banner()
username = get_username()

async def __initiate__(domain):
    try:
        async with aiohttp.ClientSession() as session:
            tasks = [abuseipdb(domain, session, args),alienvault(domain, session, args), anubis(domain, session, args), bevigil(domain, session, configpath, username, args), binaryedge(domain, session, configpath, username, args),
                     bufferover(domain, session, configpath, username, args), builtwith(domain, session, configpath, username, args),c99(domain, session, configpath, username, args),censys(domain, session, configpath, username, args), certspotter(domain, session, configpath, username, args), chaos(domain, session, configpath, username, args),coderog(domain, session, configpath, username, args),
                     columbusapi(domain, session, args), commoncrawl(domain, args),crtsh(domain, session, args), cyfare(domain, session, args),digitorus(domain, session, args), fofa(domain,session, configpath, username, args),dnsdumpster(domain, session, configpath, username, args), dnsrepo(domain, session, args),
                     facebook(domain, session, configpath, username, args), fullhunt(domain, session, configpath, username, args), google(domain, session, configpath, username, args),hackertarget(domain, session, args), huntermap(domain, session, configpath, username, args), 
                     intelx(domain, session, configpath, username, args), leakix(domain, session, configpath, username, args), merklemap(domain, args),myssl(domain, session, args),netlas(domain, session, configpath, username, args), passivetotal(domain, session, configpath, username, args),quake(domain, session, configpath, username, args),
                     rapidapi(domain, session, configpath, username, args), rapiddns(domain, session, args), rapidfinder(domain, session, configpath, username, args), rapidscan(domain, session, configpath, username, args),redhuntlabs(domain, session, configpath, username, args) ,racent(domain, session, args) ,rsecloud(domain, session, configpath, username, args) ,securitytrails(domain, session, configpath, username, args),
                     shodan(domain, session, configpath, username, args), shodanx(domain, session, args), shrewdeye(domain, session, args), sitedossier(domain, session, args), subdomaincenter(domain, session, args),trickest(domain, configpath, username, args),
                     urlscan(domain, session, args), virustotal(domain, session, configpath, username, args), waybackarchive(domain, args), whoisxml(domain, session, configpath, username, args), zoomeyeapi(domain, session, configpath, username, args)]            
            results = await asyncio.gather(*tasks)
            return results
    except KeyboardInterrupt as e:
        SystemExit
    
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception handler sources: {e}, {type(e)}{reset}", file=sys.stderr)

def gitversion():
    try:
        latest = version()
        current = "v1.0.9"
        if latest == current:
            print(f"[{blue}{bold}version{reset}]:{bold}{white}subdominator current version {current} ({green}latest{reset}{bold}{white}){reset}", file=sys.stderr)
        else:
            print(f"[{blue}{bold}version{reset}]: {bold}{white}subdominator current version {current} ({red}outdated{reset}{bold}{white}){reset}", file=sys.stderr)
            
    except KeyboardInterrupt as e:
        SystemExit
    
    
def update_handler():
    try:
        if args.show_updates:
            updatelog()
            quit()
        
        current = "v1.0.9"
        pypiold = "1.0.9"
        git = version()
        
        if current == git:
            if args.no_color:
                print(f"[INFO]: Hey {username} subdominator is already in new version", file=sys.stderr)
            else:
                print(f"[{bold}{green}INFO{reset}]: {bold}{white}Hey {username} subdominator is already in new version{reset}", file=sys.stderr)
            quit()

        zipurl = getzip()
        if not zipurl:
            if args.no_color:
                print(f"[WRN]: Hey {username} failed to update subdominator please try manually from github", file=sys.stderr)
            else:
                print(f"[{bold}{green}INFO{reset}]: {bold}{white}Hey {username} failed to update subdominator please try manually from github{reset}", file=sys.stderr)
            quit()
        cache = cachedir()
        if cache:
            launch(zipurl, cache)
        else:
            launch(zipurl, os.getcwd())
        pypiversion = getverify("subdominator")
        
        if pypiold == pypiversion:
            if args.no_color:
                print(f"[WRN]: Hey {username} failed to update subdominator please try manually from github", file=sys.stderr)
            else:
                print(f"[{bold}{green}INFO{reset}]: {bold}{white}Hey {username} failed to update subdominator please try manually from github{reset}", file=sys.stderr)
            quit()
        if args.no_color:
            print(f"[INFO]: Verified the latest version of subdominator from pypi and updated from {current} --> {git}", file=sys.stderr)
        else:
            print(f"[{bold}{green}INFO{reset}]: {bold}{white}Verified the latest version of subdominator from pypi and updated from {current} --> {git}{reset}", file=sys.stderr)
        updatelog()
        quit()
                
    except KeyboardInterrupt as e:
        quit()

def show_sources():
    try:
        resources = sources()
        if not args.no_color:
            print(f"[{bold}{blue}INFO{reset}]: {bold}{white}Current Available passive resources: [{len(resources)}]{reset}", file=sys.stderr)
        else:
            print(f"[INFO]: Current Available passive resources: [{len(resources)}]", file=sys.stderr)
        
        if not args.no_color:
            print(f"[{bold}{blue}INFO{reset}]: {bold}{white}Sources marked with an * needs API key(s) or token(s) configuration to works{reset}", file=sys.stderr)
        else:
            print(f"[INFO]: Sources marked with an * needs API key(s) or token(s) configuration to works", file=sys.stderr)
        
        if not args.no_color:
            print(f"[{bold}{blue}INFO{reset}]: {bold}{white}Hey {username} you can config your api keys or token here {configpath} to work{reset}", file=sys.stderr)
        else:
            print(f"[INFO]: Hey {username} you can config your api keys or token here {configpath} to work", file=sys.stderr)
        print()
        console = Console()
        for sourced in resources:
            console.print(Markdown(sourced))
        quit()    
    except KeyboardInterrupt as e:
        SystemExit
        
        
async def _domain_handler_(domain):
    try:
        start = time.time()
        results = await __initiate__(domain)
        filtered = filters(results)
        final= set()

        for subdomain in filtered:
            if args.filter_wildcards:
                if subdomain.startswith("*."):
                    subdomain = subdomain[2:]
            if subdomain.endswith(f".{domain}") and subdomain not in final:
                print(subdomain)
                final.add(subdomain)
                if args.output:
                    file(subdomain, domain, args)
                elif args.output_directory:
                    dir(subdomain, domain, args)

        if args.output_json:
            await jsonsave(domain, final,args.output_json, args)
        
        if args.notify:
            await notify(domain, final, configpath, username, args)
            
        end = time.time()
        total_time = end - start
        if not args.silent:
            if args.no_color:
                print(f"[INFO]: Total {len(final)} subdomains found for {domain} in {total_time:.2f} seconds", file=sys.stderr)
            else:
                print(f"[{bold}{blue}INFO{reset}]: {bold}{white}Total {len(final)} subdomains found for {domain} in {total_time:.2f} seconds{reset}", file=sys.stderr)
        
            if args.no_color:
                print(f"[WISH]: Happy Hacking {username} ☠️ 🔥 🚀", file=sys.stderr)
            else:
                print(f"[{bold}{green}WISH{reset}]: {bold}{white}Happy Hacking {username} ☠️ 🔥 🚀{reset}", file=sys.stderr)

    except KeyboardInterrupt as e:
        SystemExit


async def handler():
    try:
        if args.help:
            print(banners, file=sys.stderr) #cannot be silent in help menu
            help(configpath)
            quit()
        if not args.silent:
            print(banners, file=sys.stderr)
            if not args.disable_update_check:
                gitversion()
        if args.list_source:
            show_sources()
        if args.update or args.show_updates:
            update_handler()
        if args.domain:
            if not args.silent:
                if not args.no_color:
                    print(f"[{bold}{blue}INFO{reset}]: {bold}{white}Loading provider configuration file from {configpath}{reset}", file=sys.stderr)
                else:
                    print(f"[INFO]: Loading provider configuration file from {configpath}", file=sys.stderr)
        
                if not args.no_color:
                    print(f"[{bold}{blue}INFO{reset}]: {bold}{white}Enumerating subdomain for {args.domain}{reset}", file=sys.stderr)
                else:
                    print(f"[INFO]: Enumerating subdomain for {args.domain}", file=sys.stderr)
            await _domain_handler_(args.domain)
            quit()
    
        if args.domain_list:
            if not args.silent:
                if not args.no_color:
                    print(f"[{bold}{blue}INFO{reset}]: {bold}{white}Loading provider configuration file from {configpath}{reset}", file=sys.stderr)
                else:
                    print(f"[INFO]: Loading provider configuration file from {configpath}", file=sys.stderr)
        
            domains = reader(args.domain_list, args)
            if domains:
                for domain in domains:
                    if not args.silent:
                        if not args.no_color:
                            print(f"[{bold}{blue}INFO{reset}]: {bold}{white}Enumerating subdomain for {domain}{reset}", file=sys.stderr)
                        else:
                            print(f"[INFO]: Enumerating subdomain for {domain}", file=sys.stderr)
                    await _domain_handler_(domain)
                quit()
            
        if sys.stdin.isatty():
            if not args.no_color:
                    print(f"[{bold}{red}WRN{reset}]: {bold}{white}subdominator exits due to no inputs provided, please use subdominator -h for more details", file=sys.stderr)
            else:
                    print(f"[WRN]: subdominator exits due to no inputs provided, please use subdominator -h for more details", file=sys.stderr)
            quit()  
        else:
            for domain in sys.stdin:
                if domain:
                    domain = domain.strip()
                    if not args.silent:
                        if not args.no_color:
                            print(f"[{bold}{blue}INFO{reset}]: {bold}{white}Enumerating subdomain for {domain}{reset}", file=sys.stderr)
                        else:
                            print(f"[INFO]: Enumerating subdomain for {domain}", file=sys.stderr)
                    await _domain_handler_(domain)
            quit()
    except KeyboardInterrupt as e:
        quit()
            

def main_handler():
    try:
        asyncio.run(handler())
    except KeyboardInterrupt as e:
        quit()


from colorama import Fore, Style
import aiofiles
import random
import yaml
import aiohttp
import sys

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL    

async def randomkeys(configs, username, args):
    try:
            
            async with aiofiles.open(configs, "r") as streamr:
                
                data = yaml.safe_load(await streamr.read())
                
                if "netlas" not in data:
                    
                    return
                
                rand = data.get("netlas", [])
                
                if rand is None:
                    return
                
                randomkey = random.choice(data.get("netlas", []))
                
                
                if randomkey is None:
                    
                    return
                return randomkey
                
    except yaml.YAMLError as e:
            
            if args.show_key_info: 
                
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)  
                      
                else:
                    
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                               
    except TypeError as e:
            
            pass 
        
    except KeyboardInterrupt as e:
        quit()
        
    except yaml.YAMLObject as e:
            
            if args.show_key_info: 
                
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)    
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr)
                    
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in netlas randomkey: {e}, {type(e)}{reset}", file=sys.stderr)
            

async def netlas(domain, session, configs, username, args):
    
    try:
        
        try:
            
            netlass = []
            
            getcount = f"https://app.netlas.io/api/domains_count/?q=domain:*.{domain}+AND+NOT+domain:{domain}"
            
            randomkey = await randomkeys(configs, username, args)
            
            if randomkey is None:
                
                return []
            
            getauth = {
                "accept":    "application/json",
                "X-API-Key": f"{randomkey}"
            }
            proxy = args.proxy if args.proxy else None
            async with session.get(getcount, headers=getauth, timeout=args.timeout, proxy=proxy, ssl=False) as response:
                
                responsed = await response.json()
                if response.status !=200:
                    return []
                
                counts = responsed['count']
                                
            for val in range(0, counts+10, 20):
                
                url = f"https://app.netlas.io/api/domains/?q=domain:*.{domain}+AND+NOT+domain:{domain}&source_type=include&start={val}"
                
                randomKey = await randomkeys(configs, username, args)
                
                auth = {
                "accept":    "application/json",
                "X-API-Key": f"{randomKey}"
            }
                
                async with session.get(url, headers=auth, timeout=aiohttp.ClientTimeout(total=args.timeout,connect=args.timeout,sock_read=120,sock_connect=args.timeout), proxy=proxy, ssl=False) as response:
                    
                    data = await response.json()
                    
                    if response.status != 200:
                        
                        if len(netlass)>0:
                            return netlass
                        else:
                            return []
                    for item in data['items']:
                    
                        subdomain = item['data']['domain']   
                        netlass.append(subdomain)
                          
            return netlass    
        
        except aiohttp.ServerConnectionError as e:
            
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Netlas API, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Netlas API, due to Serverside Error", file=sys.stderr)
                
        except aiohttp.ClientConnectionError as e:
            
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Netlas API, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Netlas API, due to Clientside connection Error", file=sys.stderr)
                    
        except TimeoutError as e:
               if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Netlas API, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Netlas API, due to Timeour Error", file=sys.stderr)
        
        except KeyboardInterrupt as e:
            quit()
              
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: Exception at req block of netlas: {e}, {type(e)}{reset}", file=sys.stderr)
                    
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: Exception at main block of netlas: {e}, {type(e)}{reset}", file=sys.stderr)   
        


from colorama import Fore, Style
import aiohttp
import sys

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def alienvault(domain, session,args):
    try:
        alienvaults = []
        url = f"https://otx.alienvault.com/api/v1/indicators/hostname/{domain}/passive_dns"
        proxy = args.proxy if args.proxy else None
        async with session.get(url, timeout=args.timeout, proxy=proxy, ssl=False) as response:
            if response.status != 200:
                return []
            data = await response.json()
            for entries in data['passive_dns']:
                subdomain = entries['hostname']
                if subdomain.endswith(f".{domain}"):
                    alienvaults.append(subdomain)
            return alienvaults
    except TimeoutError as e:
        if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Alienvault API, due to Timeout", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Alienvault API, due to Timeout", file=sys.stderr)
    except KeyboardInterrupt as e:
        quit()
    except aiohttp.ServerConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Alienvault API, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Alienvault API, due to Serverside Error", file=sys.stderr)
                
    except aiohttp.ClientConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Alienvault API, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Alienvault API, due to Clientside connection Error", file=sys.stderr)
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]Exception occred in alienvault: {e}, {type(e)}", file=sys.stderr)


from colorama import Fore, Style
import aiofiles
import random
import yaml
import aiohttp
import sys

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def quake(domain, session, configs, username, args):
    
    try:
        
        try:
            
            async with aiofiles.open(configs, "r") as streamr:
                
                data = yaml.safe_load(await streamr.read())
                
                if "quake" not in data:
                    
                    return 
                
                randvalue = data.get("quake", [])
                
                if randvalue is None:  
                      
                    return  
                   
            randomkey = random.choice(data.get("quake", []))
            
            if randomkey is None:
                return
            
        except yaml.YAMLError as e:
            
            if args.show_key_info: 
                
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)  
                      
                else:
                    
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                               
        except TypeError as e:
            
            pass 
        
        except KeyboardInterrupt as e:
            quit()
        
        except yaml.YAMLObject as e:
            
            if args.show_key_info: 
                
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)    
                    
                else:
                    
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr)
                    
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured at quake read block: {e}, {type(e)}{reset}", file=sys.stderr)
            
        try:
            quakes = []
            url = f"https://quake.360.net/api/v3/search/quake_service"
            data = {
                'query': f'domain: {domain}',
                'include': ['service.http.host'],
                'latest': True,
                'start': 0,
                'size': 500,
            }
            
            auth = {
                'User-Agent': 'Mozilla/5.0 (X11; Linux i686; rv:25.0) Gecko/20100101 Firefox/25.0',
                'Accept': '*/*',
                'Accept-Language': 'en',
                'Connection': 'close',
                'Content-Type': 'application/json',
                'X-Quaketoken': randomkey
            }
            proxy = args.proxy if args.proxy else None
            async with session.post(url, headers=auth, json=data, timeout=args.timeout, proxy=proxy, ssl=False) as response:
                                
                if response.status != 200:
                    if args.show_key_info:
                            if not args.no_color:
                                print(f"[{bold}{red}ALERT{reset}]: Quake blocking our request, {username} please check your api usage for this keys: {randomkey}", file=sys.stderr)
                            else:
                                print(f"[ALERT]: Quake blocking our request, {username} please check your api usage for this keys: {randomkey}", file=sys.stderr)
                    return []
                
                data = await response.json()
                                
                for entry in data['data']:
                    
                    subdomain = entry['service']['http']['host']
                    quakes.append(subdomain)
                    
                return quakes
            
        except aiohttp.ServerConnectionError as e:
            
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Quake API, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Quake API, due to Serverside Error", file=sys.stderr)
                
        except aiohttp.ClientConnectionError as e:
            
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Quake API, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Quake API, due to Clientside connection Error", file=sys.stderr) 
            
        except KeyboardInterrupt as e:
            quit()
            
        except TimeoutError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Quake API, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Quake API, due to Timeout Error", file=sys.stderr)
                    
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception at quake req: {e}, {type(e)}{reset}", file=sys.stderr)
            
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception at quake main: {e}, {type(e)}{reset}", file=sys.stderr)
    


from colorama import Fore, Style
import aiofiles
import random
import yaml
import aiohttp
import sys

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def rapidscan(domain, session, configs, username, args):
    try:
        try:
            async with aiofiles.open(configs, "r") as streamr:
                
                data = yaml.safe_load(await streamr.read())
                
                if "rapidapi" not in data:
                    
                    return 
                
                randvalue = data.get("rapidapi", [])
                
                if randvalue is None:  
                    return []
                   
            randomkey = random.choice(data.get("rapidapi", []))
            if randomkey is None:
                return []
            
        except yaml.YAMLError as e:
            
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)  
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                               
        except TypeError as e:
            pass 
        except yaml.YAMLObject as e:
            
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)    
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                    
        except KeyboardInterrupt as e:
            quit()
            
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in rapidscan reader block: {e}, {type(e)}{reset}",file=sys.stderr)
                
        try:
            rapidscans = []
            url =  "https://subdomain-scan1.p.rapidapi.com/"
            params = {"domain":f"{domain}"}
            auth = {
	        "X-RapidAPI-Key": f"{randomkey}",
	        "X-RapidAPI-Host": "subdomain-scan1.p.rapidapi.com"
            }
            proxy = args.proxy if args.proxy else None
            async with session.get(url, headers=auth, timeout=20, params=params,proxy=proxy, ssl=False) as response:
                if response.status == 403:
                    if args.show_key_info:
                        if not args.no_color:
                            print(f"[{bold}{red}ALERT{reset}]: Rapidscan blocking our request, {username} please check that you subscribed to rapidscan API service: {randomkey}", file=sys.stderr)
                        else:
                            print(f"[ALERT]: Rapidscan blocking our request, {username} please check that you subscribed to rapidscan API service: {randomkey}", file=sys.stderr)
                    return []
                
                if  response.status != 200:
                    if args.show_key_info:
                        if not args.no_color:
                        
                            print(f"[{bold}{red}ALERT{reset}]: Rapidscan blocking our request, {username} please check your api usage for this key: {randomkey}", file=sys.stderr)
                        else:
                            print(f"[ALERT]: Rapidscan blocking our request, {username} please check your api usage for this key: {randomkey}", file=sys.stderr)
                    return []
                
                data = await response.json()
                for subdomain in data:
                    rapidscans.append(subdomain)
                return rapidscans
        
        except aiohttp.ServerConnectionError as e:
            
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Rapidscan API, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Rapidscan API, due to Serverside Error", file=sys.stderr)
                
        except aiohttp.ClientConnectionError as e:
            
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Rapidscan API, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Rapidscan API, due to Clientside connection Error", file=sys.stderr) 
        
        except KeyboardInterrupt as e:
            quit()
                           
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api rapidscan req block: {e}, due to: {type(e)}{reset}", file=sys.stderr)
                    
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in rapidscan main block: {e}, {type(e)}{reset}", file=sys.stderr)


#!/usr/bin/env python3
import random 
import os  
from colorama import Fore,Style
import requests
import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
requests.packages.urllib3.disable_warnings()

red =  Fore.RED
green = Fore.GREEN
magenta = Fore.MAGENTA
cyan = Fore.CYAN
mixed = Fore.RED + Fore.BLUE
blue = Fore.BLUE
yellow = Fore.YELLOW
white = Fore.WHITE
reset = Style.RESET_ALL
bold = Style.BRIGHT
colors = [ green, cyan, blue]
random_color = random.choice(colors)

def version():
    
    url = f"https://api.github.com/repos/Revoltsecurities/Subdominator/releases/latest"
    
    try:
        response = requests.get(url, verify=True, timeout=10)
        if response.status_code == 200:
            data = response.json()
            latest = data.get('tag_name')
            return latest
    except KeyboardInterrupt as e:
        quit()
    except Exception as e:
        pass


def sources():
    source = ["[***abuseipDB***](https://abuseipdb.com)","[***alienvault***](https://otx.alienvault.com)", "[***anubis***](https://jldc.me/anubis)", "[***bevigil****](https://bevigil.com/login)", "[***binaryedge****](https://binaryedge.io)", "[***bufferover****](https://tls.bufferover.run/)", "[***builtwith****](https://api.builtwith.com/domain-api)","[***c99****](https://subdomainfinder.c99.nl/)","[***censys****](https://censys.com/)", "[***certspotter****](https://sslmate.com/certspotter/)", "[***chaos****](https://chaos.projectdiscovery.io/)", "[**c*olumbusapi***](https://columbus.elmasy.com/)", "[***commoncrawl***](https://index.commoncrawl.org/)","[***crtsh***](https://crt.sh)", "[***cyfare***](https://cyfare.net)","[***digitorus***](https://www.digitorus.com/)", "[***dnsdumpster****](https://dnsdumpster.com/)", "[***dnsrepo***](https://dnsrepo.noc.org)", "[***fofa****](https://en.fofa.info/)","[***facebook****](https://developers.facebook.com/)", "[***fullhunt****](https://fullhunt.io/)", "[***google****](https://programmablesearchengine.google.com/controlpanel/create)","[***hackertarget***](https://hackertarget.com/)", "[***huntermap****](https://hunter.how/)", "[***intelx****](https://intelx.io/)", "[***leakix****](https://leakix.net/)", "[***merklemap***](https://www.merklemap.com)","[***myssl***](https://myssl.com)","[***netlas****](https://netlas.io/)","[***passivetotal****](https://api.riskiq.net/api/pdns_pt/#/)" ,"[***quake****](https://quake.360.cn/)", "[***racent***](https://face.racent.com)","[***rapidapi****](https://rapidapi.com/hub)", "[***rapiddns***](https://rapiddns.io/)","[***redhuntlabs****](https://devportal.redhuntlabs.com/)", "[***rsecloud****](https://rsecloud.com/search)","[***securitytrails****](http://securitytrails.com/)", "[***shodan****](https://shodan.io)", "[***shodanx***](https://github.com/RevoltSecurities/Shodanx)", "[***shrewdeye***](https://github.com/RevoltSecurities/Subdominator)", "[***sitedossier***](https://sitedossier.com/)", "[***subdomaincenter***](https://www.subdomain.center/)", "[***trickest****](https://trickest.io/)","[***urlscan***](https://urlscan.io/)", "[***virustotal****](https://virustotal.com/)", "[***waybackarchive***](https://archive.org/wayback)", "[***whoisxml****](https://whoisxmlapi.com)","[***zoomeyeapi****](https://docs.projectdiscovery.io/tools/subfinder/install#zoomeye-api)", "[***rapidfinder****](https://rapidapi.com/Glavier/api/subdomain-finder3/pricing): **Rapidfinder requires rapidapi api key but before it required to subscribe for free and please see here: https://rapidapi.com/Glavier/api/subdomain-finder3/pricing**", "[***rapidscan****](https://rapidapi.com/sedrakpc/api/subdomain-scan1/pricing): **Rapidscan requires rapidapi api key but before it required to subscribe for free and please see here: https://rapidapi.com/sedrakpc/api/subdomain-scan1/pricing**", "[***coderog****](https://rapidapi.com/coderog-coderog-default/api/subdomain-finder5/pricing): **Coderog source required to subscribe for free and please see here: https://rapidapi.com/coderog-coderog-default/api/subdomain-finder5/pricing**"]
    return source


from colorama import Fore, Style
import aiofiles
import random
import yaml
import aiohttp
import sys

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def reader(service, configs, username,args):
    try:
            async with aiofiles.open(configs, "r") as streamr:
                
                data = yaml.safe_load(await streamr.read())
                
                if f"{service}" not in data:
                    return 
                
                randvalue = data.get(f"{service}", [])
                
                if randvalue is None:  
                    return
                   
            randomkey = random.choice(data.get(f"{service}", []))
            return randomkey
            
    except yaml.YAMLError as e:
            
            if args.show_key_info: 
                
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)  
                      
                else:
                    
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                               
    except TypeError as e:
            
        pass 
        
    except yaml.YAMLObject as e:
            
        if args.show_key_info: 
            if not args.no_color: 
                print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)    
            else:
                print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                    
    except KeyboardInterrupt as e:
        quit()
            
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in notify reader block: {e}, {type(e)}{reset}",file=sys.stderr)
            
async def slack(domain, subdomains, url, session, args):
    try:
        results = '\n'.join(subdomains)
        payloads = {"text": f"Total subdomains {len(subdomains)} found for {domain}:\n{results}"}
        async with session.post(url, json=payloads) as response:
            pass
    except aiohttp.ClientConnectionError as e:
        pass
    except TimeoutError as e:
        pass
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in slack sender: {e}, {type(e)}{reset}", file=sys.stderr)
            

async def pushbullet(domain, subdomains, randomkey, session, args):
    try:
        url = f'https://api.pushbullet.com/v2/pushes'  
        results = '\n'.join(subdomains)
        headers = {
            'Access-Token': randomkey,
            
            'Content-Type': 'application/json'
            }
        jdata = {
            'type': 'note',
            'title': f"Total subdomains {len(subdomains)} found for {domain}:",
            'body': f"{results}"
        }
        async with session.post(url, headers=headers, json=jdata) as response:
            pass
        
    except aiohttp.ClientConnectionError as e:
        pass
    except TimeoutError as e:
        pass
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in slack sender: {e}, {type(e)}{reset}", file=sys.stderr)
            
            
async def notify(domain, subdomains, configpath, username, args):
    try:
        
        url = await reader("slack", configpath, username, args)
        randomkey = await reader("pushbullet", configpath, username, args)
        
        async with aiohttp.ClientSession() as session:
            if url:
                await slack(domain, subdomains, url,session, args)
            if randomkey:
                await pushbullet(domain, subdomains, randomkey, session, args)
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}exception in notify: {e}, {type(e)}{reset}", file=sys.stderr)


from colorama import Fore, Style
import aiofiles
import random
import yaml
import aiohttp
import sys

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def certspotter(domain, session, configs, username, args):
    
    try:
        
        try:
            
            async with aiofiles.open(configs, "r") as streamr:
                
                data = yaml.safe_load(await streamr.read())
                
                if "certspotter" not in data:
                    
                    return [] 
                
                randvalue = data.get("certspotter", [])
                
                if randvalue is None:  
                      
                    return []
                   
            randomkey = random.choice(data.get("certspotter", []))
            
        except yaml.YAMLError as e:
            
            if args.show_key_info: 
                
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)  
                      
                else:
                    
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                               
        except TypeError as e:
            
            pass 
        
        except yaml.YAMLObject as e:
            
            if args.show_key_info: 
                
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)    
                    
                else:
                    
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
        
        except KeyboardInterrupt as e:
            quit()
            
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in certspotter reader block: {e}, {type(e)}{reset}", file=sys.stderr)
                    
        if randomkey is None:
        
            return []
        
        try:
            
            certspotters = []
            
            url = f"https://api.certspotter.com/v1/issuances?domain={domain}&include_subdomains=true&expand=dns_names"
            
            auth = {"Authorization": f"Bearer {randomkey}"}
            proxy = args.proxy if args.proxy else None
            async with session.get(url, headers=auth, timeout=args.timeout, proxy=proxy, ssl=False) as response:
                
                data = await response.json()
                
            if response.status != 200:
                
                return []
            
            for entry in data :
                    
                dns_names = entry.get('dns_names',[])
                    
                for dns_name in dns_names:
                        
                    if dns_name.startswith("*.") and dns_name.endswith(f".{domain}"):
                            
                        subdomain = dns_name[2:]
                            
                        certspotters.append(subdomain)
                        
                    elif dns_name.endswith(f".{domain}"):
                            
                        certspotters.append(dns_name)
                        
                    else:
                            pass
                        
            return certspotters
        
        except aiohttp.ServerConnectionError as e:
            
            if args.show_timeout_info:
                 
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Certspotter API, due to Serverside Error", file=sys.stderr)    
                    
                else:
                    
                    print(f"[INFO]: Timeout reached for Certspotter API, due to Serverside Error", file=sys.stderr)
                
        except aiohttp.ClientConnectionError as e:
            
            if args.show_timeout_info:
                 
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Certspotter API, due to ClientSide connection Error", file=sys.stderr)    
                    
                else:
                    
                    print(f"[INFO]: Timeout reached for Certspotter API, due to Clientside connection Error", file=sys.stderr) 
             
        except KeyboardInterrupt as e:
            quit()
            
        except TimeoutError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Certspotter API, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Certspotter API, due to Timeout Error", file=sys.stderr)
                    
        except Exception as e:
            if args.sec_deb:    
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api certspotter req block: {e}, due to: {type(e)}{reset}", file=sys.stderr)
            
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api certpotter main block: {e}, due to: {type(e)}{reset}", file=sys.stderr)



import re
from fake_useragent import UserAgent
from colorama import Fore, Style
import aiohttp
import sys
from subdominator.modules.utils import check_subdomain
bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def abuseipdb(domain, session, args):
    try:
        parsed_domain = check_subdomain(domain)
        if parsed_domain.subdomain:
            return []
        url = f"https://www.abuseipdb.com/whois/{domain}" 
        proxy = args.proxy if args.proxy else None    
        headers = {
                     "User-Agent": UserAgent().random,
                    "Cookie": "abuseipdb_session="
        }
        async with session.get(url, timeout=10, headers=headers, proxy=proxy, ssl=False) as response:
            data = await response.text()
            if response.status != 200:
                return []
            tags = re.findall(r'<li>\w.*</li>', data)
            subdomains = [re.sub("</?li>", "", tag) + f".{domain}" for tag in tags]
            return subdomains
    except TimeoutError as e:
        if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Abuseipdb API, due to Timeout", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Abuseipdb API, due to Timeout", file=sys.stderr)      
    except aiohttp.ServerConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Abuseipdb API, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Abuseipdb API, due to Serverside Error", file=sys.stderr)            
    except aiohttp.ClientConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Abuseipdb API, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Alienvault API, due to ClientSide connection Error", file=sys.stderr)
    except KeyboardInterrupt as e:
        quit()
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in abuseipdb: {e}, {type(e)}{reset}", file=sys.stderr)



def filters(results):
    filtered = []
    for subdomains in results:
        if subdomains is None:
            subdomains = []
        for subdomain in subdomains:
            if subdomain is not None:
                filtered.append(subdomain)
    return sorted(set(filtered))



import appdirs
from colorama import Fore, Back, Style
import os
import sys

white = Fore.WHITE
blue = Fore.BLUE
red = Fore.RED
bold = Style.BRIGHT
reset = Style.RESET_ALL

yamls = """bevigil: []          

binaryedge: []

bufferover: []

builtwith: []

c99: []

censys: []

certspotter: []

chaos: []

dnsdumpster: []

facebook: []

fofa: []

fullhunt: []

google: []

huntermap: []

intelx: []

leakix: []

netlas: []

passivetotal: []

quake: []

rapidapi: []

redhuntlabs: []

rsecloud: []

virustotal: []

securitytrails: []

shodan: []

trickest: []

whoisxmlapi: []

zoomeyeapi: []

# for notifications
slack: []

pushbullet: []

# subdominator follow same syntax of key configuration of subfinder follows but some resource will be different so please look on github page of subdominator for keys setup.
"""

def get_username():
    try:
        username = os.getlogin()
    except OSError:
        username = os.getenv('USER') or os.getenv('LOGNAME') or os.getenv('USERNAME') or 'User'
    except Exception as e:
        username = "User"
    return username


def config(): 
    try:
        username = get_username()
        get_config = appdirs.user_config_dir()
        subdominator_dir = f"{get_config}/Subdominator"
        filename = "provider-config.yaml"
        config_path = f"{subdominator_dir}/{filename}"
        
        if os.path.exists(subdominator_dir):
            if os.path.exists(config_path):
                return config_path
            else:
                with open(config_path, "w") as w:
                    w.write(yamls)
                return config_path
        else:
            os.makedirs(subdominator_dir)
            with open(config_path, "w") as w:
                w.write(yamls)
            return config_path
    except Exception as e:
        print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured at config module: {e}{reset}", file=sys.stderr)

def cachedir():
    try:
        cachedir = appdirs.user_cache_dir()
        return cachedir
    except Exception as e:
        pass

def custompath(args):
    
    try:
        if os.path.exists(args.config_path) and os.path.isfile(args.config_path):
            return args.config_path
        else:
            if args.no_color:
                print(f"[WRN]: please check the the config file exists")
                quit()
            else:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}please check the the config path exists{reset}")
    except KeyboardInterrupt as e:
        quit()


import os
import json
import aiofiles
import asyncio
import sys
from subdominator.modules.utils import red, blue, white, bold, reset

def file(subdomain,  domain, args):
    try:
        if args.output:
            if os.path.isdir(args.output):
                filename = os.path.join(args.output, f"{domain}.subdomains.txt")
            else:
                filename = args.output
        if not args.output:
            filename = f"{domain}.txt"
        with open(filename, "a") as w:
            w.write(subdomain + '\n')
    except KeyboardInterrupt as e:        
        SystemExit

    except Exception as e:
        pass
        

def dir(subdomain,  domain, args):
    try:
        
        if os.path.isdir(args.output_directory):
            if os.path.exists(args.output_directory):
                filename = f"{args.output_directory}/{domain}.txt"
            else:
                os.makedirs(args.output_directory)
                filename = f"{args.output_directory}/{domain}.txt"
        else:
            currentdir = os.getcwd()
            filename = f"{currentdir}/{domain}.txt"
            
        with open(filename, "a") as w:
            w.write(subdomain + '\n')
    except KeyboardInterrupt as e:        
        SystemExit
    except Exception as e:
        pass

async def jsonsave(domain, subdomains, filename, args):
    try:
        if args.output_json:
            if os.path.isdir(args.output_json):
                filename = os.path.join(args.output_json, f"{domain}.subdomains.json")
            else:
                filename = args.output_json
        results = []
        for subdomain in subdomains:
            results.append({
                        "subdomain": subdomain,
                        "domain": domain
                    })
        async with aiofiles.open(filename, "a") as streamw:
            for output in results:
                await streamw.write(json.dumps(output)+"\n")
    except Exception as e:
        print(f"[{bold}{red}WRN{reset}]: {bold}{white}Excepiton occured in json output writer due to: {e}, {type(e)}{reset}")
        


from colorama import Fore, Style
import aiofiles
import random
import yaml
import aiohttp
import sys
import asyncio
from fake_useragent import UserAgent
bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def randomkeys(configs, username, args):
    try:
            async with aiofiles.open(configs, "r") as streamr:
                data = yaml.safe_load(await streamr.read())
                
                if "zoomeyeapi" not in data:
                    return
                
                rand = data.get("zoomeyeapi", [])
                if rand is None:
                    return
                
                randomkey = random.choice(data.get("zoomeyeapi", []))
                
                if randomkey is None:
                    return
                return randomkey
                
    except yaml.YAMLError as e:
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)  
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                               
    except TypeError as e:
        pass 
        
    except yaml.YAMLObject as e:
        if args.show_key_info: 
            if not args.no_color: 
                print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)    
            else:
                print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr)
                              
    except Exception as e:
        if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in zoomeyeapi reader block: {e}, {type(e)}{reset}", file=sys.stderr)
    
async def region(domain, session,randomkey, args): 
    
    try:
        url = f"https://api.zoomeye.org/domain/search?q={domain}&type=1&s=1000&page=1"
        
        auth = {
            "API_KEY": f"{randomkey}",
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        
        async with session.get(url, headers=auth, timeout=aiohttp.ClientTimeout(total=args.timeout,connect=args.timeout,sock_read=120,sock_connect=args.timeout)) as response:
            return response.status                 
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in zooeye region block: {e}, {type(e)}{reset}", file=sys.stderr)

async def zoomeyeapi(domain, session , configs, username, args):
            
        try: 
            
            zoomeyes = []
            
            testkey = await randomkeys(configs, username, args)
            
            status = await region(domain, session,testkey, args)
            if not status:
                return zoomeyes
            page =1
            
            while True:
                if status != 403:
                    url = f"https://api.zoomeye.org/domain/search?q={domain}&type=1&s=1000&page={page}" # for china users
                else:
                    url = f"https://api.zoomeye.hk/domain/search?q={domain}&type=1&s=1000&page={page}"  # for other countries users
                    
                randomkey = await randomkeys(configs, username, args)
                if not randomkey:
                    if len(zoomeyes)>0:
                        return zoomeyes
                    return
                    
                auth = {
                    "API-KEY": f"{randomkey}",
                    "User-Agent": f"{UserAgent().random}"
                }
                proxy = args.proxy if args.proxy else None
                async with session.get(url, headers=auth, timeout=aiohttp.ClientTimeout(total=args.timeout,connect=args.timeout,sock_read=120,sock_connect=args.timeout), proxy=proxy,ssl=False) as response:
                    
                    data = await response.json()

                    if response.status != 200:
                        if len(zoomeyes)>0:
                            return zoomeyes
                        return
                    
                    if "list" not in data:
                        if args.show_key_info:
                            if not args.no_color:
                                print(f"[{bold}{red}ALERT{reset}]: Zoomeye blocking our request, {username} please check your api usage for this key: {randomkey}", file=sys.stderr)
                            else:
                                print(f"[ALERT]: Zoomeye blocking our request, {username} please check your api usage for this key: {randomkey}", file=sys.stderr)
                        if len(zoomeyes)>0:
                            return zoomeyes
                        return
                    
                    for item in data["list"]:
                        zoomeyes.append(item["name"])
                                
                    page +=1
                    
        except aiohttp.ServerConnectionError as e:
            
            if args.show_timeout_info:
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Zoomeye API, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Zoomeye API, due to Serverside Error", file=sys.stderr)
                    
                
        except aiohttp.ClientConnectionError as e:
            
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Zoomeye API, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Zoomeye API, due to Clientside connection Error", file=sys.stderr) 
                    
        except KeyboardInterrupt as e:
            quit()        
            
        except TimeoutError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Zoomeye API, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Zoomeye API, due to Timeout Error", file=sys.stderr)
                           
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in zoomeye request block: {e}, {type(e)}{reset}", file=sys.stderr)


from colorama import Fore, Style
import re
import httpx
import sys
from fake_useragent import UserAgent
from subdominator.modules.utils import compiler, extracts

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL
Waybackurls = set()

async def waybackarchive(domain, args):
    try:
       headers = {
           "User-Agent": UserAgent().random
       }
       proxy = args.proxy if args.proxy else None
       async with httpx.AsyncClient(verify=False, proxy=proxy) as request:
           async with request.stream("GET", f"https://web.archive.org/cdx/search/cdx?url=*.{domain}&collapse=urlkey&fl=original", headers=headers, timeout=httpx.Timeout(read=300.0, connect=args.timeout, write=None, pool=None), follow_redirects=True) as response:
               async for url in response.aiter_lines():
                   subdomains = await extracts(url, domain)
                   if subdomains:
                       for subdomain in subdomains:
                           subdomain = subdomain.lstrip("25").lstrip("2F").lstrip("40").lstrip(".").lstrip("B0")
                           if subdomain not in Waybackurls and not subdomain.startswith("%3D") and not subdomain.startswith("3D"):
                               Waybackurls.add(subdomain)
    except KeyboardInterrupt as e:
        quit()
    except TimeoutError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Webarchive, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Webarchive, due to Timeout Error", file=sys.stderr)
    except httpx.ConnectTimeout:
        if args.show_timeout_info:
            if not args.no_color: 
                print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Webarchive, due to Timeout Error", file=sys.stderr)    
            else:
                print(f"[INFO]: Timeout reached for Webarchive, due to Timeout Error", file=sys.stderr)
    except httpx.ConnectError:
        if args.show_timeout_info:
            if not args.no_color: 
                print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Webarchive, due to Timeout Error", file=sys.stderr)    
            else:
                print(f"[INFO]: Timeout reached for Webarchive, due to Timeout Error", file=sys.stderr)
    except httpx.ReadTimeout:
        pass
    except Exception as e:
        if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in wayback request block: {e}, {type(e)}{reset}", file=sys.stderr)
    finally:
        return Waybackurls


from colorama import Fore, Style
import aiohttp
import sys
from fake_useragent import UserAgent

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def crtsh(domain, session, args):
    try:
        url = f"https://crt.sh/?q=%25.{domain}&output=json"
        headers = {
            "User-Agent": UserAgent().random
        }
        proxy = args.proxy if args.proxy else None
        async with session.get(url, timeout=aiohttp.ClientTimeout(total=args.timeout,connect=args.timeout,sock_read=300,sock_connect=args.timeout), headers=headers ,proxy=proxy, ssl=False) as response:
            if response.status != 200:
                return []
            data = await response.json()
            subdomains = [domain for domains in data for domain in domains['name_value'].split('\n')]
            return subdomains
        
    except aiohttp.ServerConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Crtsh, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Crtsh, due to Serverside Error", file=sys.stderr)  
    except KeyboardInterrupt as e:
        quit()
    except TimeoutError as e:
        if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Crtsh, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Crtsh, due to Timeout Error", file=sys.stderr)
                    
    except aiohttp.ClientConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Crtsh, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Crtsh, due to Clientside connection Error", file=sys.stderr)
                    
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception at crtsh : {e}, {type(e)}{reset}", file=sys.stderr)


from colorama import Fore, Style
import aiofiles
import random
import yaml
import aiohttp
import sys
from fake_useragent import UserAgent

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def randomkeys(configs, username, args):
    try:
            async with aiofiles.open(configs, "r") as streamr:
                data = yaml.safe_load(await streamr.read())
                if "google" not in data:
                    return
                
                rand = data.get("google", [])
                if rand is None:
                    return
                randomkey = random.choice(data.get("google", []))
                
                cx, key = randomkey.split(":")
                return cx, key
                
    except yaml.YAMLError as e:
            
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)  
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                               
    except TypeError as e:
            pass 
        
    except KeyboardInterrupt as e:
        quit()
        
    except yaml.YAMLObject as e:
            
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)    
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr)
                    
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in google randomkey: {e}, {type(e)}{reset}", file=sys.stderr)
            
            
async def google(domain, session, configs, username, args):
    try:
        googles = []
        page = 1
        while True:
            randomcx, randomkey = await randomkeys(configs, username, args)

            if randomcx is None or randomkey is None:
                return googles
            url = f"https://customsearch.googleapis.com/customsearch/v1?q=site:*.{domain}%20-www&cx={randomcx}&num=10&start={page}&key={randomkey}&alt=json"
            headers = {
                "User-Agent": UserAgent().random
            }
            proxy=args.proxy if args.proxy else None
            async with session.get(url, headers=headers, timeout=args.timeout, proxy=proxy, ssl=False) as response:
                if response.status != 200:
                    return googles
                data = await response.json()
                for item in data.get("items", []):
                    subdomains = item.get("displayLink")
                    if subdomains:
                        googles.append(subdomains)
            page+=1
    except aiohttp.ServerConnectionError as e:
            
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Google API, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Google API, due to Serverside Error", file=sys.stderr)
                
    except aiohttp.ClientConnectionError as e:
            
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Google API, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Google API, due to Clientside connection Error", file=sys.stderr)
                    
    except TimeoutError as e:
               if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Google API, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Google API, due to Timeour Error", file=sys.stderr)
    except KeyError as e:
        pass 
    except TypeError as e:
        pass
    except KeyboardInterrupt as e:
            quit()
              
    except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: Exception at req block of google: {e}, {type(e)}{reset}", file=sys.stderr)



from colorama import Fore, Style
import aiofiles
import random
import yaml
import aiohttp
import sys

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def fullhunt(domain, session, configs, username, args):
    
    try:
        
        try:
            
            async with aiofiles.open(configs, "r") as streamr:
                
                data = yaml.safe_load(await streamr.read())
                
                if "fullhunt" not in data:
                    return []
                
                rand = data.get("fullhunt", [])
                
                if rand is None:
                    return []
                
                randomkey = random.choice(data.get("fullhunt", []))
                
                if randomkey is None:
                    return []
        except yaml.YAMLError as e:
            
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)  
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                               
        except TypeError as e:
            pass 
        
        except KeyboardInterrupt as e:
            quit()
        
        except yaml.YAMLObject as e:
            
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)    
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr)
                    
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}INFO{reset}]: {bold}{white}Exception in fullhunt reader block: {e}, {type(e)}{reset}", file=sys.stderr)
                
        try:
            
            fullhunts = []
            
            url = f"https://fullhunt.io/api/v1/domain/{domain}/subdomains"
            
            auth = {"X-API-KEY": f"{randomkey}"}
            
            proxy = args.proxy if args.proxy else None
            
            async with session.get(url, headers=auth, timeout=aiohttp.ClientTimeout(total=args.timeout,connect=args.timeout,sock_read=120,sock_connect=args.timeout), proxy=proxy, ssl=False) as response:
                data = await response.json()
                if response.status != 200:
                    if args.show_key_info:
                        if not args.no_color:
                        
                            print(f"[{bold}{red}ALERT{reset}]: Fullhunt blocking our request, {username} please check your api usage for this key: {randomkey}", file=sys.stderr)  
                        else:    
                            print(f"[ALERT]: Fullhunt blocking our request, {username} please check your api usage for this key: {randomkey}", file=sys.stderr)
                    return []
                
                subdomains = data.get('hosts', [])   
                for subdomain in subdomains:
                    fullhunts.append(subdomain)
                    
                return fullhunts
            
        except aiohttp.ServerConnectionError as e:
            
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Fullhunt API, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Fullhunt API, due to Serverside Error", file=sys.stderr)
                
        except aiohttp.ClientConnectionError as e:
            
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Fullhunt API, due to ClientSide connection Error", file=sys.stderr)    
                    
                else:
                    print(f"[INFO]: Timeout reached for Fullhunt API, due to Clientside connection Error", file=sys.stderr) 
        
        except TimeoutError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Fullhunt API, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Fullhunt API, due to Timeout Error", file=sys.stderr)
        
        except KeyboardInterrupt as e:
            quit() 
                           
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}INFO{reset}]: {bold}{white}Exception occured in api fullhunt req block: {e}, due to: {type(e)}{reset}", file=sys.stderr)
            
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}INFO{reset}]: {bold}{white}Exception occured in fullhunt main block: {e}, {type(e)}{reset}", file=sys.stderr)